<?php
/**
 * Awe Menu is quick, easy to setup and WYSIWYG menu management system
 *
 * Awe Menu by Kahanit(https://www.kahanit.com) is licensed under a
 * Creative Creative Commons Attribution-NoDerivatives 4.0 International License.
 * Based on a work at https://www.kahanit.com.
 * Permissions beyond the scope of this license may be available at https://www.kahanit.com.
 * To view a copy of this license, visit http://creativecommons.org/licenses/by-nd/4.0/.
 *
 * @author    Amit Sidhpura <amit@kahanit.com>
 * @copyright 2016 Kahanit
 * @license   http://creativecommons.org/licenses/by-nd/4.0/
 * @version   1.0.5.0
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'awemenu/config/define.php';
include_once _MODULE_AM_DIR_ . 'libraries/kahanit/Helpers.php';

/**
 * Class AweMenu
 */
class AweMenu extends Module
{
    public static $css_styles = array();

    public static $css_styles_counter = 1;

    public function __construct()
    {
        $this->name = 'awemenu';
        $this->tab = 'front_office_features';
        $this->version = '1.0.5';
        $this->author = 'Kahanit';
        $this->need_instance = 0;
        $this->module_key = '8af6d7ada1fca473058ec50262c2e860';

        parent::__construct();

        $this->displayName = $this->l('Awe Menu - PrestaShop Top Mega Menu Module');
        $this->description = $this->l('Quick, easy to setup and WYSIWYG menu management module');
        $this->confirmUninstall = $this->l('Are you sure?');
    }

    public function install()
    {
        include _MODULE_AM_DIR_ . 'sql/install.php';

        if (!parent::install()
            || !$this->registerHook('header')
            || !$this->registerHook('displayTop')

            || !$this->registerHook('actionObjectProductUpdateAfter')
            || !$this->registerHook('actionObjectCategoryUpdateAfter')
            || !$this->registerHook('actionObjectCmsUpdateAfter')
            || !$this->registerHook('actionObjectManufacturerUpdateAfter')
            || !$this->registerHook('actionObjectSupplierUpdateAfter')

            || !$this->registerHook('actionObjectProductDeleteAfter')
            || !$this->registerHook('actionObjectCategoryDeleteAfter')
            || !$this->registerHook('actionObjectCmsDeleteAfter')
            || !$this->registerHook('actionObjectManufacturerDeleteAfter')
            || !$this->registerHook('actionObjectSupplierDeleteAfter')

            || !KIHelpers::installModuleTab('Awe Menu', 'AdminAweMenu', $this->name, 'AdminParentPreferences', true)
        ) {
            return false;
        }

        return true;
    }

    public function uninstall()
    {
        $this->clearMenuCache();
        include _MODULE_AM_DIR_ . 'sql/uninstall.php';

        if (!parent::uninstall()
            || !$this->unregisterHook('header')
            || !$this->unregisterHook('displayTop')

            || !$this->unregisterHook('actionObjectProductUpdateAfter')
            || !$this->unregisterHook('actionObjectCategoryUpdateAfter')
            || !$this->unregisterHook('actionObjectCmsUpdateAfter')
            || !$this->unregisterHook('actionObjectManufacturerUpdateAfter')
            || !$this->unregisterHook('actionObjectSupplierUpdateAfter')

            || !$this->unregisterHook('actionObjectProductDeleteAfter')
            || !$this->unregisterHook('actionObjectCategoryDeleteAfter')
            || !$this->unregisterHook('actionObjectCmsDeleteAfter')
            || !$this->unregisterHook('actionObjectManufacturerDeleteAfter')
            || !$this->unregisterHook('actionObjectSupplierDeleteAfter')

            || !KIHelpers::uninstallModuleTab('AdminAweMenu')
        ) {
            return false;
        }

        return true;
    }

    public function getContent()
    {
        $link = $this->context->link->getAdminLink('AdminAweMenu');
        Tools::redirectAdmin($link);
        die();
    }

    public function hookHeader()
    {
        KIHelpers::getLibrary('Menu', $this);
        $model_menu = new AMMenuModel();
        $menu = $model_menu->getLiveMenu($this->context->shop->id);

        if (!$this->isCached('display_top/menu.tpl', $this->getCacheId())) {
            self::validateMenu($menu['menu']);
            self::generateMenuCSS($menu['menu']);

            $css = "/* autogenerated styles starts */\n";

            foreach (self::$css_styles as $class_name => $css_style) {
                $css .= "#awemenu #awemenu-forpriority ." . $class_name . '{' . $css_style . "}\n";
            }

            $css .= $menu['theme']->css;
            $css .= "\n/* autogenerated styles ends */";

            file_put_contents(_MODULE_AM_DIR_ . 'views/css/menu-custom-' . $menu['id'] . '.css', $css);

            $this->smarty->assign(array(
                'menu'    => $menu['menu'],
                'lang_id' => $this->context->language->id
            ));
        }

        $this->context->controller->addCSS($this->_path . 'views/css/fontawesome.min.css');
        $this->context->controller->addCSS($this->_path . 'views/css/menu-layout.css');
        $this->context->controller->addCSS($this->_path . 'views/css/menu-responsive.css');
        $this->context->controller->addCSS($this->_path . 'views/css/menu-theme-' . $menu['id'] . '.css');
        $this->context->controller->addCSS($this->_path . 'views/css/menu-custom-' . $menu['id'] . '.css');

        $this->context->controller->addJquery();
        $this->context->controller->addJS($this->_path . 'views/js/jquery.awemenu.front.js');
    }

    public function hookDisplayTop()
    {
        return $this->display(__FILE__, 'display_top/menu.tpl', $this->getCacheId());
    }

    public static function generateMenuCSS($items = array())
    {
        if (!is_array($items) || count($items) == 0) {
            return false;
        }

        foreach ($items as $item) {
            $css_properties = array();

            if (isset($item->config->color) && !empty($item->config->color)) {
                $css_properties['color'] = $item->config->color;
            }

            if (isset($item->config->width) && !empty($item->config->width)) {
                $css_properties['width'] = $item->config->width;
            }

            if (isset($item->config->height) && !empty($item->config->height)) {
                $css_properties['height'] = $item->config->height;
            }

            if (isset($item->config->{'background-color'}) && !empty($item->config->{'background-color'})) {
                $css_properties['background-color'] = $item->config->{'background-color'};
            }

            if (isset($item->config->{'background-image'}) && !empty($item->config->{'background-image'})) {
                $css_properties['background-image'] = $item->config->{'background-image'};
            }

            if (isset($item->config->{'background-repeat'}) && !empty($item->config->{'background-repeat'})) {
                $css_properties['background-repeat'] = $item->config->{'background-repeat'};
            }

            if (isset($item->config->{'background-position'}) && !empty($item->config->{'background-position'})) {
                $css_properties['background-position'] = $item->config->{'background-position'};
            }

            if (isset($item->config->{'font-size'}) && !empty($item->config->{'font-size'})) {
                $css_properties['font-size'] = $item->config->{'font-size'};
            }

            if (isset($item->config->bold) && !empty($item->config->bold)) {
                $css_properties['font-weight'] = (($item->config->bold) ? 'bold' : '');
            }

            if (isset($item->config->italic) && !empty($item->config->italic)) {
                $css_properties['font-style'] = (($item->config->italic) ? 'italic' : '');
            }

            if (isset($item->config->underline) && !empty($item->config->underline)) {
                $css_properties['text-decoration'] = (($item->config->underline) ? 'underline' : '');
            }

            $css_style = '';

            $css_style .= implode(';', array_map(function ($v, $k) {
                if ($k == 'background-image') {
                    return sprintf('%s:url(%s)', $k, $v);
                } else {
                    return sprintf('%s:%s', $k, $v);
                }
            },
                $css_properties,
                array_keys($css_properties)));

            if ($css_style != '') {
                if (in_array($css_style, self::$css_styles)) {
                    $item->config->{'class-autogen'} = array_search($css_style, self::$css_styles);
                } else {
                    self::$css_styles['am-autogenclass-' . self::$css_styles_counter] = $css_style;
                    $item->config->{'class-autogen'} = 'am-autogenclass-' . self::$css_styles_counter;
                    self::$css_styles_counter++;
                }
            }

            // badge styles
            $css_style = '';

            if (isset($item->config->{'badge-color'}) && !empty($item->config->{'badge-color'})) {
                $css_style .= 'color:' . $item->config->{'badge-color'} . ';';
            }

            if (isset($item->config->{'badge-background-color'}) && !empty($item->config->{'badge-background-color'})) {
                $css_style .= 'background-color:' . $item->config->{'badge-background-color'} . ';';
            }

            if ($css_style != '') {
                if (in_array($css_style, self::$css_styles)) {
                    $item->config->{'class-badge-autogen'} = array_search($css_style, self::$css_styles);
                } else {
                    self::$css_styles['am-autogenclass-' . self::$css_styles_counter] = $css_style;
                    $item->config->{'class-badge-autogen'} = 'am-autogenclass-' . self::$css_styles_counter;
                    self::$css_styles_counter++;
                }
            }

            // dropdown styles
            $css_style = '';

            if (isset($item->config->dropdown->width) && !empty($item->config->dropdown->width)) {
                $css_style .= 'width:' . $item->config->dropdown->width . ';';
            }

            if (isset($item->config->dropdown->height) && !empty($item->config->dropdown->height)) {
                $css_style .= 'height:' . $item->config->dropdown->height . ';';
            }

            if (isset($item->config->dropdown->{'background-color'}) && !empty($item->config->dropdown->{'background-color'})) {
                $css_style .= 'background-color:' . $item->config->dropdown->{'background-color'} . ';';
            }

            if (isset($item->config->dropdown->{'background-image'}) && !empty($item->config->dropdown->{'background-image'})) {
                $css_style .= 'background-image:url(\'' . $item->config->dropdown->{'background-image'} . '\');';
            }

            if (isset($item->config->dropdown->{'background-repeat'}) && !empty($item->config->dropdown->{'background-repeat'})) {
                $css_style .= 'background-repeat:' . $item->config->dropdown->{'background-repeat'} . ';';
            }

            if (isset($item->config->dropdown->{'background-position'}) && !empty($item->config->dropdown->{'background-position'})) {
                $css_style .= 'background-position:' . $item->config->dropdown->{'background-position'} . ';';
            }

            if ($css_style != '') {
                if (in_array($css_style, self::$css_styles)) {
                    $item->config->{'class-dropdown-autogen'} = array_search($css_style, self::$css_styles);
                } else {
                    self::$css_styles['am-autogenclass-' . self::$css_styles_counter] = $css_style;
                    $item->config->{'class-dropdown-autogen'} = 'am-autogenclass-' . self::$css_styles_counter;
                    self::$css_styles_counter++;
                }
            }

            $item->items = self::generateMenuCSS($item->items);
        }

        return $items;
    }

    public static function validateMenu($items = array())
    {
        if (!is_array($items) || count($items) == 0) {
            return false;
        }

        $context = Context::getContext();
        $id_lang = $context->language->id;
        $id_lang_default = Configuration::get('PS_LANG_DEFAULT');

        foreach ($items as $item) {
            if ($item->xtype == 'tab' || $item->xtype == 'link') {
                if (isset($item->config->shop_id)) {
                    $id_shop = $item->config->shop_id;
                } else {
                    $id_shop = $context->shop->id;
                }
                switch ($item->config->entity) {
                    case 'product':
                        $product = new Product($item->config->entity_item_id, false, $id_lang, $id_shop);
                        $item->config->url = (object)array();

                        if (isset($product->id) && !empty($product->id)) {
                            $item->config->text->{'lang_id_' . $id_lang} = $product->name;
                            $item->config->url->{'lang_id_' . $id_lang} = $context->link->getProductLink($item->config->entity_item_id, null, null, null, $id_lang, $id_shop);
                        } else {
                            $item->config->url->{'lang_id_' . $id_lang} = '#';
                        }

                        break;
                    case 'category':
                        $category = new Category($item->config->entity_item_id, $id_lang, $id_shop);
                        $item->config->url = (object)array();

                        if (isset($category->id) && !empty($category->id)) {
                            $item->config->text->{'lang_id_' . $id_lang} = $category->name;
                            $item->config->url->{'lang_id_' . $id_lang} = $context->link->getCategoryLink($item->config->entity_item_id, null, $id_lang, null, $id_shop);
                        } else {
                            $item->config->url->{'lang_id_' . $id_lang} = '#';
                        }

                        break;
                    case 'cmspage':
                        $cmspage = new CMS($item->config->entity_item_id, $id_lang, $id_shop);
                        $item->config->url = (object)array();

                        if (isset($cmspage->id) && !empty($cmspage->id)) {
                            $item->config->text->{'lang_id_' . $id_lang} = $cmspage->meta_title;
                            $item->config->url->{'lang_id_' . $id_lang} = $context->link->getCMSLink($item->config->entity_item_id, null, null, $id_lang, $id_shop);
                        } else {
                            $item->config->url->{'lang_id_' . $id_lang} = '#';
                        }

                        break;
                    case 'manufacturer':
                        $manufacturer = new Manufacturer($item->config->entity_item_id, $id_lang);
                        $item->config->url = (object)array();

                        if (isset($manufacturer->id) && !empty($manufacturer->id)) {
                            $item->config->text = $manufacturer->name;
                            $item->config->url = $context->link->getManufacturerLink($item->config->entity_item_id, null, $id_lang, $id_shop);
                        } else {
                            $item->config->url = '#';
                        }

                        break;
                    case 'supplier':
                        $supplier = new Supplier($item->config->entity_item_id, $id_lang);
                        $item->config->url = (object)array();

                        if (isset($supplier->id) && !empty($supplier->id)) {
                            $item->config->text = $supplier->name;
                            $item->config->url = $context->link->getSupplierLink($item->config->entity_item_id, null, $id_lang, $id_shop);
                        } else {
                            $item->config->url = '#';
                        }

                        break;
                    case 'custom':
                        if (!isset($item->config->text->{'lang_id_' . $id_lang}) || empty($item->config->text->{'lang_id_' . $id_lang})) {
                            $item->config->text->{'lang_id_' . $id_lang} = $item->config->text->{'lang_id_' . $id_lang_default};
                        }

                        if (!isset($item->config->url->{'lang_id_' . $id_lang}) || empty($item->config->url->{'lang_id_' . $id_lang})) {
                            $item->config->url->{'lang_id_' . $id_lang} = $item->config->url->{'lang_id_' . $id_lang_default};
                        }

                        break;
                }

                if (!isset($item->config->{'custom-text'}->{'lang_id_' . $id_lang}) || empty($item->config->{'custom-text'}->{'lang_id_' . $id_lang})) {
                    $item->config->{'custom-text'}->{'lang_id_' . $id_lang} = $item->config->{'custom-text'}->{'lang_id_' . $id_lang_default};
                }

                if (!isset($item->config->{'badge-text'}->{'lang_id_' . $id_lang}) || empty($item->config->{'badge-text'}->{'lang_id_' . $id_lang})) {
                    $item->config->{'badge-text'}->{'lang_id_' . $id_lang} = $item->config->{'badge-text'}->{'lang_id_' . $id_lang_default};
                }
            }

            if ($item->xtype == 'html' && (!isset($item->config->content->{'lang_id_' . $id_lang}) || empty($item->config->content->{'lang_id_' . $id_lang}))) {
                $item->config->content->{'lang_id_' . $id_lang} = $item->config->content->{'lang_id_' . $id_lang_default};
            }

            $item->items = self::validateMenu($item->items);
        }

        return $items;
    }

    public function clearMenuCache()
    {
        $this->_clearCache('display_top/menu.tpl');
    }

    public function hookActionObjectProductUpdateAfter()
    {
        $this->_clearCache('display_top/menu.tpl');
    }

    public function hookActionObjectCategoryUpdateAfter()
    {
        $this->_clearCache('display_top/menu.tpl');
    }

    public function hookActionObjectCmsUpdateAfter()
    {
        $this->_clearCache('display_top/menu.tpl');
    }

    public function hookActionObjectManufacturerUpdateAfter()
    {
        $this->_clearCache('display_top/menu.tpl');
    }

    public function hookActionObjectSupplierUpdateAfter()
    {
        $this->_clearCache('display_top/menu.tpl');
    }

    public function hookActionObjectProductDeleteAfter()
    {
        $this->_clearCache('display_top/menu.tpl');
    }

    public function hookActionObjectCategoryDeleteAfter()
    {
        $this->_clearCache('display_top/menu.tpl');
    }

    public function hookActionObjectCmsDeleteAfter()
    {
        $this->_clearCache('display_top/menu.tpl');
    }

    public function hookActionObjectManufacturerDeleteAfter()
    {
        $this->_clearCache('display_top/menu.tpl');
    }

    public function hookActionObjectSupplierDeleteAfter()
    {
        $this->_clearCache('display_top/menu.tpl');
    }
}
