<?php
/**
 * Awe Menu is quick, easy to setup and WYSIWYG menu management system
 *
 * Awe Menu by Kahanit(https://www.kahanit.com) is licensed under a
 * Creative Creative Commons Attribution-NoDerivatives 4.0 International License.
 * Based on a work at https://www.kahanit.com.
 * Permissions beyond the scope of this license may be available at https://www.kahanit.com.
 * To view a copy of this license, visit http://creativecommons.org/licenses/by-nd/4.0/.
 *
 * @author    Amit Sidhpura <amit@kahanit.com>
 * @copyright 2016 Kahanit
 * @license   http://creativecommons.org/licenses/by-nd/4.0/
 * @version   1.0.5.0
 */

require_once _PS_MODULE_DIR_ . 'awemenu/config/define.php';
include_once _MODULE_AM_DIR_ . 'libraries/kahanit/Helpers.php';
include_once _MODULE_AM_DIR_ . 'libraries/autoload.php';

/**
 * Class AMMenu
 */
class AdminAweMenuController extends ModuleAdminController
{
    public $id_shop = 0;

    public function init()
    {
        parent::init();

        KIHelpers::shopRedirect(true);
        $this->bootstrap = true;

        $ajax = (int)Tools::getValue('ajax', 0);
        if ($ajax) {
            $this->id_shop = (int)Tools::getValue('id_shop');
        } else {
            $this->id_shop = (int)Shop::getContextShopID();
        }
    }

    public function setMedia()
    {
        parent::setMedia();

        KIHelpers::getLibrary('Menu', $this->module);
        $model_menu = new AMMenuModel();
        $menu = $model_menu->getEditorMenu($this->id_shop);

        $this->addCSS(_MODULE_AM_URI_ . 'views/css/fontawesome.min.css');
        $this->addCSS(_MODULE_AM_URI_ . 'views/css/glyphicon.min.css');
        $this->addCSS(_MODULE_AM_URI_ . 'views/css/dataTables.bootstrap.min.css');
        $this->addCSS(_MODULE_AM_URI_ . 'views/css/ui.fancytree.min.css');
        $this->addCSS(_MODULE_AM_URI_ . 'views/css/bootstrap-colorpicker.min.css');
        $this->addCSS(_MODULE_AM_URI_ . 'views/css/fontawesome-iconpicker.min.css');
        $this->addCSS(_MODULE_AM_URI_ . 'views/css/menu-layout.css');
        $this->addCSS(_MODULE_AM_URI_ . 'views/css/menu-responsive.css');
        $this->addCSS(_MODULE_AM_URI_ . 'views/css/menu-editor.css');
        $this->addCSS(_MODULE_AM_URI_ . 'views/css/menu-theme-' . $menu['id'] . '.css');
        $this->addCSS(_MODULE_AM_URI_ . 'views/css/admin.css');

        $this->addJquery();
        $this->addJqueryUI('ui.sortable');
        $this->addJqueryUI('effects.highlight');
        $this->addJS(_MODULE_AM_URI_ . '/views/js/handlebars.min.js');
        $this->addJS(_MODULE_AM_URI_ . '/views/js/jquery.dataTables.min.js');
        $this->addJS(_MODULE_AM_URI_ . '/views/js/dataTables.bootstrap.min.js');
        $this->addJS(_MODULE_AM_URI_ . '/views/js/jquery.fancytree-all.min.js');
        $this->addJS(_MODULE_AM_URI_ . '/views/js/bootstrap-colorpicker.min.js');
        $this->addJS(_MODULE_AM_URI_ . '/views/js/fontawesome-iconpicker.min.js');
        $this->addJS(_MODULE_AM_URI_ . '/views/js/ace.js');
        $this->addJS(_PS_JS_DIR_ . 'tiny_mce/tiny_mce.js');
        $this->addJS(_PS_JS_DIR_ . 'admin/tinymce.inc.js');
        $this->addJS(_MODULE_AM_URI_ . '/views/js/jquery.awemenu.loader.js');
    }

    public function initContent()
    {
        parent::initContent();

        // get languages
        $langs = Language::getLanguages(false);
        $alllangs = array();
        $activeLang = $this->context->language;

        foreach ($langs as $lang) {
            $alllangs[] = (object)array(
                'id'   => $lang['id_lang'],
                'name' => $lang['name'],
                'iso'  => $lang['iso_code']
            );
        }

        // get shops
        $shops = Shop::getShops();
        $allshops = array();

        foreach ($shops as $shop) {
            $allshops[] = (object)array(
                'id'   => $shop['id_shop'],
                'name' => $shop['name']
            );
        }

        $this->context->smarty->assign(array(
            'module_name'         => $this->module->name,
            'module_display_name' => $this->module->displayName,
            'module_version'      => $this->module->version,
            // for awemenu variables
            'url'                 => 'index.php?controller=AdminAweMenu&token=' . Tools::getValue('token') . '&ajax=1&id_shop=' . $this->id_shop,
            'module_url'          => _MODULE_AM_URI_,
            'js_url'              => _PS_JS_DIR_,
            'js_module_url'       => _MODULE_AM_URI_ . 'views/js/',
            'langs'               => Tools::jsonEncode($alllangs),
            'activelang'          => $activeLang->id,
            'shops'               => $allshops,
            'activeshop'          => $this->id_shop,
            'entities'            => Tools::jsonEncode(array(
                (object)array('entity' => 'product', 'name' => 'Products', 'format' => 'table'),
                (object)array('entity' => 'category', 'name' => 'Categories', 'format' => 'tree'),
                (object)array('entity' => 'cmspage', 'name' => 'CMS Pages', 'format' => 'table'),
                (object)array('entity' => 'manufacturer', 'name' => 'Manufacturers', 'format' => 'table'),
                (object)array('entity' => 'supplier', 'name' => 'Suppliers', 'format' => 'table')
            )),
            // for tinymce variables
            'iso'                 => file_exists(_PS_CORE_DIR_ . '/js/tiny_mce/langs/' . $activeLang->iso_code . '.js') ? $activeLang->iso_code : 'en',
            'path_css'            => _THEME_CSS_DIR_,
            'ad'                  => __PS_BASE_URI__ . basename(_PS_ADMIN_DIR_)
        ));

        $this->setTemplate('menu.tpl');
    }

    public function displayAjax()
    {
        KIHelpers::displayAjax($this);
    }

    public function getThemeCssfromLess($params)
    {
        $parser = new Less_Parser();
        $parser->parseFile(_MODULE_AM_DIR_ . 'views/less/menu-theme.less');
        $parser->ModifyVars(array(
            'mb-topclr'     => '#' . $params['mb-topclr'],
            'mb-btmclr'     => '#' . $params['mb-btmclr'],
            'mb-txtclr'     => '#' . $params['mb-txtclr'],
            'mb-hvr-topclr' => '#' . $params['mb-hvr-topclr'],
            'mb-hvr-btmclr' => '#' . $params['mb-hvr-btmclr'],
            'mb-hvr-txtclr' => '#' . $params['mb-hvr-txtclr'],
            'mb-bdrclr'     => '#' . $params['mb-bdrclr'],
            'dd-topclr'     => '#' . $params['dd-topclr'],
            'dd-btmclr'     => '#' . $params['dd-btmclr'],
            'dd-txtclr'     => '#' . $params['dd-txtclr'],
            'dd-ttlclr'     => '#' . $params['dd-ttlclr'],
            'dd-bdrclr'     => '#' . $params['dd-bdrclr']
        ));
        $css = $parser->getCss();

        header("Content-type: text/css; charset: UTF-8");

        return $css;
    }

    public function getEditorMenu()
    {
        KIHelpers::getLibrary('Menu', $this->module);
        $model_menu = new AMMenuModel();

        return Tools::jsonEncode($model_menu->getEditorMenu($this->id_shop));
    }

    public function getRevisions()
    {
        KIHelpers::getLibrary('Menu', $this->module);
        $model_menu = new AMMenuModel();

        return Tools::jsonEncode(array(
            'recordsTotal'    => $model_menu->getNumRevisions($this->id_shop),
            'recordsFiltered' => $model_menu->getNumRevisions($this->id_shop),
            'data'            => $model_menu->getRevisions($this->id_shop)
        ));
    }

    public function saveMenu()
    {
        $menu = Tools::jsonDecode(urldecode(Tools::getValue('menu', '{}')));
        $menu->shop = $this->id_shop;
        $menu->menu = KIHelpers::addslashes(Tools::jsonEncode($menu->menu));
        $theme = $menu->theme;
        $menu->theme = KIHelpers::addslashes(Tools::jsonEncode($menu->theme));

        // save menu
        KIHelpers::getLibrary('Menu', $this->module);
        $model_menu = new AMMenuModel();
        $menu_id = $model_menu->saveMenu($menu);

        // save css
        $parser = new Less_Parser();
        $parser->parseFile(_MODULE_AM_DIR_ . 'views/less/menu-theme.less');
        $parser->ModifyVars(array(
            'mb-topclr'     => '#' . $theme->{'mb-topclr'},
            'mb-btmclr'     => '#' . $theme->{'mb-btmclr'},
            'mb-txtclr'     => '#' . $theme->{'mb-txtclr'},
            'mb-hvr-topclr' => '#' . $theme->{'mb-hvr-topclr'},
            'mb-hvr-btmclr' => '#' . $theme->{'mb-hvr-btmclr'},
            'mb-hvr-txtclr' => '#' . $theme->{'mb-hvr-txtclr'},
            'mb-bdrclr'     => '#' . $theme->{'mb-bdrclr'},
            'dd-topclr'     => '#' . $theme->{'dd-topclr'},
            'dd-btmclr'     => '#' . $theme->{'dd-btmclr'},
            'dd-txtclr'     => '#' . $theme->{'dd-txtclr'},
            'dd-ttlclr'     => '#' . $theme->{'dd-ttlclr'},
            'dd-bdrclr'     => '#' . $theme->{'dd-bdrclr'}
        ));
        $css = $parser->getCss();
        file_put_contents(_MODULE_AM_DIR_ . 'views/css/menu-theme-' . $menu_id . '.css', $css);
        $this->module->clearMenuCache();

        return $this->getEditorMenu();
    }

    public function editRevision($params = array())
    {
        KIHelpers::getLibrary('Menu', $this->module);
        $model_menu = new AMMenuModel();

        $model_menu->editRevision($params['id']);

        return $this->getEditorMenu();
    }

    public function liveRevision($params = array())
    {
        KIHelpers::getLibrary('Menu', $this->module);
        $model_menu = new AMMenuModel();
        $this->module->clearMenuCache();

        return $model_menu->liveRevision($params['id']);
    }

    public function deleteRevision($params = array())
    {
        KIHelpers::getLibrary('Menu', $this->module);
        $model_menu = new AMMenuModel();
        $this->module->clearMenuCache();
        $model_menu->deleteRevision($params['id']);

        return $this->getEditorMenu();
    }

    public function getEntity($params = array())
    {
        switch ($params['entity']) {
            case 'product':
                return $this->getProducts();
            case 'category':
                return $this->getCategories();
            case 'cmspage':
                return $this->getCMSPages();
            case 'manufacturer':
                return $this->getManufacturers();
            case 'supplier':
                return $this->getSuppliers();
        }

        return false;
    }

    public function getProducts()
    {
        $order = Tools::getValue('order', array(array('column' => 0, 'dir' => 'ASC')));
        $columns = Tools::getValue('columns', array());

        $start = (int)Tools::getValue('start', 0);
        $length = (int)Tools::getValue('length', 25);
        $category_id = (int)Tools::getValue('id_category', null);
        $search = Tools::getValue('search', '');

        if (isset($columns[$order[0]['column']]['data'])) {
            $orderfld = Tools::strtolower($columns[$order[0]['column']]['data']);
        } else {
            $orderfld = '';
        }

        if (isset($order[0]['dir'])) {
            $orderdir = Tools::strtoupper($order[0]['dir']);
        } else {
            $orderdir = '';
        }

        $langs = Language::getLanguages(false);

        KIHelpers::getLibrary('kahanit/Products', $this->module);
        $products = KIProducts::getProducts($this->context->language->id, $category_id, $search, $start, $length, $orderfld, $orderdir, $this->id_shop);
        $products_total = KIProducts::getNumProducts($category_id, $search, $this->id_shop);

        $data = array();
        $c = 0;

        foreach ($products as $product) {
            $data[$c] = array();

            $data[$c]['entity'] = 'product';
            $data[$c]['entity_item_id'] = $product['id_product'];
            $data[$c]['shop_id'] = $this->id_shop;

            foreach ($langs as $lang) {
                $product_lang = new Product($product['id_product'], false, $lang['id_lang']);

                $data[$c]['text']['lang_id_' . $lang['id_lang']] = $product_lang->name;
            }

            $data[$c] = (object)$data[$c];
            $c++;
        }

        return Tools::jsonEncode(array(
            'recordsTotal'    => $products_total,
            'recordsFiltered' => $products_total,
            'data'            => $data
        ));
    }

    public function getCategories()
    {
        $lang_id = $this->context->language->id;
        $id_category_parent = (int)Tools::getValue('parent_id', Category::getRootCategory($lang_id, (new Shop($this->id_shop, $lang_id, $this->id_shop)))->id);
        $search = trim(Tools::getValue('search', ''));

        $langs = Language::getLanguages(false);

        KIHelpers::getLibrary('kahanit/Categories', $this->module);
        $categories = KICategories::getCategories($lang_id, $id_category_parent, $search, $this->id_shop);

        $data = array();
        $c = 0;

        foreach ($categories as $category) {
            $data[$c] = array();
            $data[$c]['key'] = $category['id_category'];
            $data[$c]['folder'] = true;

            $data[$c]['entity'] = 'category';
            $data[$c]['entity_item_id'] = $category['id_category'];
            $data[$c]['shop_id'] = $this->id_shop;

            if ($search == '') {
                $var_bool = (bool)KICategories::getChildrenCount($category['id_category']);

                $data[$c]['expanded'] = !$var_bool;
                $data[$c]['lazy'] = $var_bool;
            } else {
                $data[$c]['expanded'] = true;
                $data[$c]['lazy'] = false;
            }

            foreach ($langs as $lang) {
                $category_lang = new Category($category['id_category'], $lang['id_lang']);

                $data[$c]['text']['lang_id_' . $lang['id_lang']] = $category_lang->name;
            }

            $data[$c]['title'] = $data[$c]['text']['lang_id_' . $lang_id];

            $data[$c] = (object)$data[$c];
            $c++;
        }

        return Tools::jsonEncode($data);
    }

    public function getCMSPages()
    {
        $order = Tools::getValue('order', array(array('column' => 0, 'dir' => 'ASC')));
        $columns = Tools::getValue('columns', array());

        $start = (int)Tools::getValue('start', 0);
        $length = (int)Tools::getValue('length', 25);
        $search = Tools::getValue('search', '');

        if (isset($columns[$order[0]['column']]['data'])) {
            $orderfld = Tools::strtolower($columns[$order[0]['column']]['data']);
        } else {
            $orderfld = '';
        }

        if (isset($order[0]['dir'])) {
            $orderdir = Tools::strtoupper($order[0]['dir']);
        } else {
            $orderdir = '';
        }

        $langs = Language::getLanguages(false);

        KIHelpers::getLibrary('kahanit/CMSPages', $this->module);
        $cmspages = KICMSPages::getCMSPages($this->context->language->id, $search, $start, $length, $orderfld, $orderdir, $this->id_shop);
        $cmspages_total = KICMSPages::getNumCMSPages($search, $this->id_shop);

        $data = array();
        $c = 0;

        foreach ($cmspages as $cmspage) {
            $data[$c] = array();

            $data[$c]['entity'] = 'cmspage';
            $data[$c]['entity_item_id'] = $cmspage['id_cms'];
            $data[$c]['shop_id'] = $this->id_shop;

            foreach ($langs as $lang) {
                $cmspage_lang = new CMS($cmspage['id_cms'], $lang['id_lang']);

                $data[$c]['text']['lang_id_' . $lang['id_lang']] = $cmspage_lang->meta_title;
            }

            $data[$c] = (object)$data[$c];
            $c++;
        }

        return Tools::jsonEncode(array(
            'recordsTotal'    => $cmspages_total,
            'recordsFiltered' => $cmspages_total,
            'data'            => $data
        ));
    }

    public function getManufacturers()
    {
        $order = Tools::getValue('order', array(array('column' => 0, 'dir' => 'ASC')));
        $columns = Tools::getValue('columns', array());

        $start = (int)Tools::getValue('start', 0);
        $length = (int)Tools::getValue('length', 25);
        $search = Tools::getValue('search', '');

        if (isset($columns[$order[0]['column']]['data'])) {
            $orderfld = Tools::strtolower($columns[$order[0]['column']]['data']);
        } else {
            $orderfld = '';
        }

        if (isset($order[0]['dir'])) {
            $orderdir = Tools::strtoupper($order[0]['dir']);
        } else {
            $orderdir = '';
        }

        KIHelpers::getLibrary('kahanit/Manufacturers', $this->module);
        $manufacturers = KIManufacturers::getManufacturers($search, $start, $length, $orderfld, $orderdir, $this->id_shop);
        $manufacturers_total = KIManufacturers::getNumManufacturers($search, $this->id_shop);

        $data = array();
        $c = 0;

        foreach ($manufacturers as $manufacturer) {
            $data[$c] = array();

            $data[$c]['entity'] = 'manufacturer';
            $data[$c]['entity_item_id'] = $manufacturer['id_manufacturer'];
            $data[$c]['shop_id'] = $this->id_shop;
            $data[$c]['text'] = $manufacturer['name'];

            $data[$c] = (object)$data[$c];
            $c++;
        }

        return Tools::jsonEncode(array(
            'recordsTotal'    => $manufacturers_total,
            'recordsFiltered' => $manufacturers_total,
            'data'            => $data
        ));
    }

    public function getSuppliers()
    {
        $order = Tools::getValue('order', array(array('column' => 0, 'dir' => 'ASC')));
        $columns = Tools::getValue('columns', array());

        $start = (int)Tools::getValue('start', 0);
        $length = (int)Tools::getValue('length', 25);
        $search = Tools::getValue('search', '');

        if (isset($columns[$order[0]['column']]['data'])) {
            $orderfld = Tools::strtolower($columns[$order[0]['column']]['data']);
        } else {
            $orderfld = '';
        }

        if (isset($order[0]['dir'])) {
            $orderdir = Tools::strtoupper($order[0]['dir']);
        } else {
            $orderdir = '';
        }

        KIHelpers::getLibrary('kahanit/Suppliers', $this->module);
        $suppliers = KISuppliers::getSuppliers($search, $start, $length, $orderfld, $orderdir, $this->id_shop);
        $suppliers_total = KISuppliers::getNumSuppliers($search, $this->id_shop);

        $data = array();
        $c = 0;

        foreach ($suppliers as $supplier) {
            $data[$c] = array();

            $data[$c]['entity'] = 'supplier';
            $data[$c]['entity_item_id'] = $supplier['id_supplier'];
            $data[$c]['shop_id'] = $this->id_shop;
            $data[$c]['text'] = $supplier['name'];

            $data[$c] = (object)$data[$c];
            $c++;
        }

        return Tools::jsonEncode(array(
            'recordsTotal'    => $suppliers_total,
            'recordsFiltered' => $suppliers_total,
            'data'            => $data
        ));
    }
}
