<?php
/**
 * Awe Menu is quick, easy to setup and WYSIWYG menu management system
 *
 * Awe Menu by Kahanit(https://www.kahanit.com) is licensed under a
 * Creative Creative Commons Attribution-NoDerivatives 4.0 International License.
 * Based on a work at https://www.kahanit.com.
 * Permissions beyond the scope of this license may be available at https://www.kahanit.com.
 * To view a copy of this license, visit http://creativecommons.org/licenses/by-nd/4.0/.
 *
 * @author    Amit Sidhpura <amit@kahanit.com>
 * @copyright 2016 Kahanit
 * @license   http://creativecommons.org/licenses/by-nd/4.0/
 * @version   1.0.5.0
 */

/**
 * Class AMMenuModel
 */
class AMMenuModel
{
    public function getEditorMenu($id_shop = 0)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('awemenu');
        $sql->where('`deleted` = 0 AND `edit` = 1 AND `shop` = ' . (int)$id_shop);

        $row = Db::getInstance()->getRow($sql);

        if (!is_array($row) || empty($row['menu']) || empty($row['theme'])) {
            $row = $this->getEmptyMenu();
        }

        $row['menu'] = Tools::jsonDecode(KIHelpers::stripslashes($row['menu']));
        $row['theme'] = Tools::jsonDecode(KIHelpers::stripslashes($row['theme']));

        return $row;
    }

    public function getLiveMenu($id_shop = 0)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('awemenu');
        $sql->where('`deleted` = 0 AND `live` = 1 AND (`shop` = 0 OR `shop` = ' . (int)$id_shop . ')');
        $sql->orderBy('`shop` DESC');

        $row = Db::getInstance()->getRow($sql);

        if (!is_array($row) || empty($row['menu']) || empty($row['theme'])) {
            $row = $this->getEmptyMenu();
        }

        $row['menu'] = Tools::jsonDecode(KIHelpers::stripslashes($row['menu']));
        $row['theme'] = Tools::jsonDecode(KIHelpers::stripslashes($row['theme']));

        return $row;
    }

    public function getEmptyMenu()
    {
        $row = array(
            'id'      => 0,
            'title'   => 'Untitled',
            'shop'    => 0,
            'author'  => 0,
            'menu'    => Tools::file_get_contents(_MODULE_AM_DIR_ . 'sql/menu-empty.txt'),
            'theme'   => Tools::file_get_contents(_MODULE_AM_DIR_ . 'sql/theme.txt'),
            'edit'    => 1,
            'live'    => 1,
            'deleted' => 0,
            'date'    => time()
        );

        return $row;
    }

    public function getRevisions($id_shop = 0)
    {
        $sql = new DbQuery();
        $sql->select('*, DATE_FORMAT(FROM_UNIXTIME(date), \'%a %D %b, %Y %h:%i %p\') as `date`');
        $sql->from('awemenu');
        $sql->where('`deleted` = 0 AND `shop` = ' . (int)$id_shop);
        $sql->orderby('`id` DESC');

        return Db::getInstance()->executeS($sql);
    }

    public function getNumRevisions($id_shop = 0)
    {
        $sql = new DbQuery();
        $sql->select('COUNT(`id`) AS total');
        $sql->from('awemenu');
        $sql->where('`deleted` = 0 AND `shop` = ' . (int)$id_shop);

        return Db::getInstance()->getValue($sql);
    }

    public function saveMenu($menu)
    {
        if ($menu->edit == '1') {
            Db::getInstance()->update('awemenu', array(
                'edit' => 0
            ), '`shop` = ' . (int)$menu->shop);
        }

        if ($menu->live == '1') {
            Db::getInstance()->update('awemenu', array(
                'live' => 0
            ), '`shop` = ' . (int)$menu->shop);
        }

        if (Db::getInstance()->insert('awemenu', array(
            'title' => pSQL($menu->title),
            'shop'  => (int)$menu->shop,
            'menu'  => pSQL($menu->menu, true),
            'theme' => pSQL($menu->theme, true),
            'edit'  => (int)$menu->edit,
            'live'  => (int)$menu->live,
            'date'  => time()
        ))
        ) {
            return Db::getInstance()->Insert_ID();
        } else {
            return 0;
        }
    }

    public function editRevision($id = null)
    {
        if ($id == null) {
            return false;
        }

        Db::getInstance()->update('awemenu', array(
            'edit' => 0
        ), '`shop` = ' . (int)$this->getIdShopFromId($id));

        return Db::getInstance()->update('awemenu', array(
            'edit' => 1
        ), '`id` = ' . (int)$id);
    }

    public function liveRevision($id = null)
    {
        if ($id == null) {
            return false;
        }

        Db::getInstance()->update('awemenu', array(
            'live' => 0
        ), '`shop` = ' . (int)$this->getIdShopFromId($id));

        return Db::getInstance()->update('awemenu', array(
            'live' => 1
        ), '`id` = ' . (int)$id);
    }

    public function deleteRevision($id = null)
    {
        if ($id == null) {
            return false;
        }

        return Db::getInstance()->update('awemenu', array(
            'deleted' => 1
        ), '`id` = ' . (int)$id);
    }

    public function getIdShopFromId($id)
    {
        $sql = new DbQuery();
        $sql->select('`shop`');
        $sql->from('awemenu');
        $sql->where('`id` = ' . (int)$id);

        return Db::getInstance()->getValue($sql);
    }
}
